(* ::Package:: *)

(* euvec.m *)
(* Version 5. *)
(* by Rolf Sulanke *)
(* Date: March 3, 2008 *)

(* Version 1: February 24, 1999. Revised May 25, 1999. Revised October 4, 2000.
Revised January 30, 2010 *)


(* This package contains definitions and procedures for n-dimensional
Euclidean geometry in terms of vector algebra. Applications, explanations,
and certain more special concepts are included in the notebook Euklid.nb.*)

(* Keywords: vector objects, null vector, basis vectors, random vector,
normalization, cross product, stereographic projection, inverse stereographic
projection, hyperplanes, spheres, spheres through four points, hyperspheres,
orthoframes in the 4-dimensional euclidean vector space,
Erhard Schmidt's orthogonalization, orthogonal complement. *)
(*Most of the concepts depend on the dimension dim, which must be defined in
the  Global Context! *)

(* Main changes: orthoerg, orthoframe, and ortho1 are replaced by the
 more general orthocomplement,
orthobase[v] is replaced by the help of orthocomplement[{v}]  *)

(* Free software: Feel free to apply, distribute or change this package. Please,
cite the source and notice the changes, or new concepts,  which you
 introduced.*)

(*Warning. The package can not substitute the package euvec.m in the item 
Spheres, MathSource, Enhancements/Geometry 0210-092. *)


BeginPackage["euvec`",{"vectorcalc`"}]


(*The usages *)

cross::usage = "cross[v] gives the vector product of a list v of 
n - 1 vectors of the euclidean space of dimension n ."

sterproj::usage ="sterproj is the stereographical projection of the unit 
n-sphere from the north pole stb[n+1] onto the euclidean (equatorial) n-space. 
WARNING: The result is correct only for arguments of norm 1 !";

invsterproj::usage="invsterproj is the inverse of the general euclidean 
stereographic projection sterproj.";

spherpt3D::usage = "If a is a vector of dimension 4, then spherpt3D[a] is 
the graphics3D primitive Point of the stereographical projection sterproj of 
the normed vector a.";

hyperplane::usage ="hyperplane[plist] returns the normal vector n and the
distance p from the origin of  the hyperplane  of the N-dimensional euclidean
space spanned by the N points in general position listed in plist; plist must
be a square  matrix of order N."

unit::usage = "unit[n][s_List] generates a parameter representation of the unit 
n-sphere with centre o in the euclidean (n+1)-space. The parameters are
the first n elements of the list s, which must have length at least n. The
parameter ranges  0 <s[[1]]<=2Pi, -Pi/2 <= s[[j]] <= Pi/2 give the whole
n-sphere. unit[s_List] = unit[Length[s]][s_list]."

sphericalreflection::usage = "sphericalreflection[x, a, r] gives the point in
 inversion with x \
with 
respect to the hypersphere with center a and radius r."

esorthonorm::usage = "esorthonorm[b_List, opts] is the Erhard Schmidt
orthogonalization for the list b of vectors with respect to a positive
semi-definite scalar product which can be specified as an option; the
default is Dot."

norm::usage = "norm[v] is the norm of the vector v with respect to a positive
semi-definite scalar product which can be specified as an option for innerprod; the
default is Dot."

innerprod::usage = "innerprod is an option for esorthonorm, norm, and orthonorm;the
default is Dot (esorthonorm), resp. pssp (orthonorm)."

normed::usage = "normed is an option for esorthonorm and orthonorm which
 specifies whether
the vectors produced by esorthonorm or orthonorm are normed, or only
 orthogonal;the default
is True."

print::usage = "print is an option for esorthonorm and orthonorm which specifies
 whether
the (unnormed) vectors produced by esorthonorm or orthonorm are printed during
 the
orthogonalization procedure;the default is False."

neglect::usage = "neglect is an option for esorthonorm and orthonorm which
 specifies whether
vectors with components or scalar square less than 10^neglect
produced by esorthonorm  or orthonorm during the orthogonalization procedure are
 set to
zero;the default is neglect -> -10.For neglect -> 0 no neglecting takes place."

orthocomplement::usage = "In a finite-dimensional vector space provided with a 
positive definite scalar product orthocomplement[vList, opts] yields an orthonormal
basis of the orthogonal complement of the span of vList; opts are the same as to
esorthonorm." ;

(* Only for the case dim = 4 :*)

subsphere::usage = "subsphere[n,p][v,w] is the parameter  representation of the
2-sphere in the euclidean 4-space, in which  the hyperplane n.x=p intersects
the unit 3-sphere. n is a normal vector of the hyperplane,  
vp=p/Sqrt[n.n] is the distance  of the hyperplane from the origin, 
which must satisfy -1<vp<1; 
v (0 <= v < 2Pi), w  (-Pi/2 <= w <= Pi/2), are the parameters for the
2-sphere.  Its radius is Sqrt[1-vp^2], and its center is n*p/n.n."

tg4frame::usage = "tg4frame[s,t,u] is an orthonormal moving frame in the
euclidean 4-space such that the fourth vector is the radius vector (= normal
vector) of the unit 3-sphere with geographic coordinates s,t,u."

subspheremf::usage ="subspheremf[s,t,u,p][v,w] is the parameter  representation
 of
the 2-sphere in the euclidean 4-space, in which  the hyperplane n.x=p
intersects the unit 3-sphere. 
s,t,u are geographical coordinates on the unit 3-sphere, 
n=unit[3][{s,t,u}] is the normed normal vector of the hyperplane, 
p, -1<p<1, is the distance of the  hyperplane from the origin, 
v  (0 <= v < 2Pi), w  (-Pi/2 <= w <= Pi/2), are the parameters for the
2-sphere.  Its radius is Sqrt[1-p^2], and its center is n*p."

(*For Graphics3D, indepentently of the value of dim: *)
 
sphere::usage = "sphere[m,r][u,v] is the parameter representation of 
the sphere with center m and radius r in the euclidean 3-space with geographical
 
coordinates u, 0<=u<=2 Pi, v, -Pi/2 <= v <= Pi/2."

sphereplot3D::usage = "sphereplot3D[m,r, u0,u1,v0,v1,opt] is the plot of the 
part of the sphere with center m and radius r, which is defined by u0<=u<=u1, 
v0<=v<=v1; the default is the whole sphere given by the default values u0=0, 
u1=2 Pi, v0 =-Pi/2, v1=Pi/2. The options opt are transferred to
ParametricPlot3D."



(* The dimension must be set within the Global Context. *)

dimm := Global`dim;


Begin["`Private`"]
      
(*Definition of the stereographic projection of the n-sphere onto the \
\
n-dimensional euclidean space: *)

sterproj[v_List] :=Module[{m=Length[v],l, w},
		l=null[m];
		w=v[[m]];
		If[smoothing[w - 1]== 0,For[i=1,i<m,i++,l[[i]]=Infinity],
		l=Simplify[ stb[m,m] + (v- stb[m,m])/(1- w)],
		l=Simplify[ stb[m,m] + (v- stb[m,m])/(1- w)]];
		l=Take[l,m-1];
		l]


(*The inverse stereographic projection*)

invsterproj[y_List]:= Module[{n=Length[y]+1,x=Append[y,0], u=y.y},
		x*2/(1+u) +stb[n,n]*(u-1)/(u+1)]


spherpt3D[a_] := Point[sterproj[dotnorme[a]]]

(* A private version of the Euclidean cross product for arbitrary dimension:*)

cross[b_List] := Module[{d = Length[b] + 1,v},
		v = null[d];
			For[i=1,i<d,i++,If[Length[b[[i]] ]!= d, 
			Print[b[[i]], " is not a correct argument!"];Return[],
			v[[i]] = Det[Join[b, {stb[i, d]}]]]];
			v[[d]] = Det[Join[b, {stb[d, d]}]];
			Return[v]]

hyperplane[pt_List] := Module[{n,q,p,dd=Length[pt], fin},
		If[dd == 0, Goto[fin]];
		For[i=1,i<= dd,i++, If[Length[pt[[i]]] != dd, 
		Print["Length of ", pt[[i]], " is unequal to ",dd ];Goto[fin]]];
		q = Table[pt[[i]] - pt[[dd]], {i,dd - 1}];
			n = cross[q];
	If[n === null[dd], 
	Print["The points ", pt, " are not in general position!"];Goto[fin]];
n = n/Sqrt[n.n];
p = n.pt[[1]];		
Return[{n, p}];
	Label[fin];
 Print["Invalid arguments!"];	
Return[]]

(* Inversions at hyperspheres *)

sphericalreflection[{Infinity}, a_List,r_:1] := a;
sphericalreflection[x_List, a_List, r_:1] := If[Not[x==a],
a + ((x - a) * r^2)/((x-a).(x-a)),{Infinity},
a + ((x - a) * r^2)/((x-a).(x-a))]

(*A fast Erhard Schmidt orthonormalization procedure; may be applied also for
functional spaces, the inner product of which must be positive semi-definite. *)


Options[esorthonorm] = {innerprod ->Dot,normed -> True, print ->False, 
neglect-> -10}

esorthonorm[m_List, opts___] := 
  Module[
		{ma = m, mb = {}, k = Length[m], rd = Length[m], j = 0, i, ne, prod, pr, 
      bn, sm, aus, h, f},
		{prod, nor, pr, bn} = 
			{innerprod, normed, print, neglect} /. {opts} /. Options[esorthonorm];
    If[Head[m[[1]] ] == List, rd = Length[m[[1]]]];
	f[h_,bn_:-10]:=If[ bn == 0, h, Chop[h,10^bn]];
	sm[aus_] := MapAll[f, aus];
		Label[ne];
    If[Length[mb] >= Min[rd, k], Return[mb]];
		j = j + 1;
    If[j > k, Return[mb]];
			ma[[j]] = sm[Simplify[ma[[j]] -
	Sum[mb[[i]]*prod[ma[[j]], mb[[i]]]/prod[mb[[i]], mb[[i]]], 
	{i, 1, Length[mb]}]]]; 		
If[pr == True, Print[ma[[j]]]];
If[prod[ma[[j]], ma[[j]]] == 0, Goto[ne],
        If[nor == True, 
        AppendTo[mb, sm[Simplify[ma[[j]]/Sqrt[prod[ma[[j]], ma[[j]]]]]]],
				AppendTo[mb, Simplify[ma[[j]]]]],
				If[nor == True,
				AppendTo[mb, sm[Simplify[ma[[j]]/Sqrt[prod[ma[[j]], ma[[j]]]]]]], 			
AppendTo[mb, Simplify[ma[[j]]]]]];		
		Goto[ne]];

Options[norm] ={innerprod -> Dot};

norm[v_, opts___] := Module[{prod},
{prod} = {innerprod}/.{opts}/.Options[norm];
Sqrt[prod[v,v]]];

orthocomplement[v_List,opts___]:=Module[{ve= v,d, veo,rk,bas, vm,vef,veff,i},
If[Not[MatrixQ[ve]],Print["Incorrect argument!"];Return[]];
d= Length[ve[[1]]];
veo=esorthonorm[ve,opts];
rk = Length[veo];
bas = Table[stb[i,d],{i,d}];
vm  = Join[veo,bas];
vef = esorthonorm[vm, opts];
veff =Table[vef[[i]],{i,rk+1,d}];
Return[veff]]


(* The following definitions work in case dim = 4 only! *)

subsphere[n_List, p_][v_,w_] := Module[{fr,pa,nv,pv,s,err},
	s=Simplify[Sqrt[n.n]];
	If[Length[n]!= 4 || s == 0, Goto[err]];	
	nv = n/s;
	pv = p/s;
	If[Abs[pv]> 1, Goto[err]];
	fr = orthocomplement[{nv}];
	pa := nv*pv +
				((fr[[1]]*Cos[v] + fr[[2]]*Sin[v])*Cos[w] +
							fr[[3]]*Sin[w])*Sqrt[1 - pv^2];
		Return[pa];
		Label[err];
		Print[ n, " or ", p,":  incorrect arguments!"]; Return[]]

tg4frame[s_,t_,u_] :={{Sin[s],-Cos[s],0,0},
			{-Cos[s]*Sin[t],-Sin[s]*Sin[t], Cos[t], 0},
		{-Cos[s] Cos[t] Sin[u],-Cos[t] Sin[s] Sin[u],-Sin[t] Sin[u],Cos[u]},{Cos[s] \
\
Cos[t] Cos[u],Cos[t] Cos[u] Sin[s],Cos[u] Sin[t],Sin[u]}}

subspheremf[s_,t_,u_, p_][v_,w_] := Module[{fr,pa},
	If[Abs[p]>1, Print["Parameter p = ",p," is too large!"];Return[]];
		fr = tg4frame[s,t,u];
			pa := fr[[4]] *p +
				((fr[[1]]*Cos[v] + fr[[2]]*Sin[v])*Cos[w] +
							fr[[3]]*Sin[w])*Sqrt[1 - p^2];
		Return[pa]]


(*Recursive Definition of the parameter representation of the unit n-sphere*)

(*This definition is independent of a globally defined dimension dim. *)

unit[n_][s_List] := Module[{un,i},
		un[1]={Cos[s[[1]]], Sin[s[[1]]]};
			For[i=2, i<= n, i++,
		un[i]=AppendTo[un[i-1],0]*Cos[s[[i]]]+stb[1+i,i+1]*Sin[s[[i]]]];
				Return[un[n]]]/;Length[s]>=n
				
unit[s_List] := unit[Length[s]][s]

(*Spheres in the euclidean 3-space *)

sphere[m_List, r_][u_, v_] := {m[[1]] + r*Cos[u]*Cos[v], 
    m[[2]] + r*Cos[v]*Sin[u], m[[3]] + r*Sin[v]} /; Length[m] == 3
    
sphereplot3D[m_List, r_, u0_:0, u1_:2 Pi,v0_:-Pi/2,v1_:Pi/2, opt___] := 
  ParametricPlot3D[sphere[m,r][u,v],{u,u0,u1},{v,v0,v1},opt]

End[]

Protect[cross, esorthonorm, innerprod,  normed,
neglect,  print, sterproj, invsterproj, 
 orthocomplement, sphere, sphereplot3D, subsphere,
 subspheremf, 
sphericalreflection, tg4frame, hyperplane, unit]

EndPackage[]


